from collections import namedtuple

import networkx as nx
from evrouting.T import Wh, ChargingCoefficient, Time, Node, NodeData, EdgeData

TemplateEdge = namedtuple('Edge', ['u', 'v', 'distance', 'consumption'])
TemplateNode = namedtuple(
    'Node', ['label', 'charging_coeff'], defaults=(None, None)
)

LABEL_KEY = 'label'
CHARGING_COEFFICIENT_KEY = 'c'
DISTANCE_KEY = 'weight'
CONSUMPTION_KEY = 'c'


def node_convert(n: TemplateNode) -> NodeData:
    return {LABEL_KEY: n.label, CHARGING_COEFFICIENT_KEY: n.charging_coeff}


def edge_convert(e: TemplateEdge) -> EdgeData:
    return {DISTANCE_KEY: e.distance, CONSUMPTION_KEY: e.consumption}


def consumption(G: nx.Graph, u: Node, v: Node) -> Wh:
    return G.edges[u, v][CONSUMPTION_KEY]


def distance(G: nx.Graph, u: Node, v: Node) -> Time:
    return G.edges[u, v][DISTANCE_KEY]


def charging_cofficient(G: nx.Graph, n: Node) -> ChargingCoefficient:
    return G.nodes[n][CHARGING_COEFFICIENT_KEY]


def label(G: nx.Graph, u: Node) -> str:
    return G.nodes[u][LABEL_KEY]
