from typing import Tuple

import networkx as nx

from evrouting.osm.const import ms_to_kmh
from evrouting.osm.imports import haversine_distance

lat = float
lon = float
point = Tuple[lat, lon]


def find_nearest(G, v: point):
    min_dist = None
    closest_node = None

    lat_v, lon_v = v

    for n in G.nodes:
        d = haversine_distance(
            G.nodes[n]['lat'],
            G.nodes[n]['lot'],
            lat_v,
            lon_v,
            unit_m=True
        )
        if min_dist is None or d < min_dist:
            closest_node = n
            min_dist = d

    return closest_node


def shortest_path(G, s: point, t: point, profile):
    """Calc A* shortest path."""
    _s = find_nearest(G, s)
    _t = find_nearest(G, t)

    def dist(u, v):
        return haversine_distance(
            G.nodes[u]['lat'],
            G.nodes[u]['lot'],
            G.nodes[v]['lat'],
            G.nodes[v]['lon'],
            unit_m=True
        ) / profile['maxspeed'] * ms_to_kmh

    return nx.astar_path(G, _s, _t, heuristic=dist)
